% Simulation of the bacterial chemotaxis model given in
% Barkai, Leibler, Nature, 1997.
clear all

% Simulation time [s]
Tf=180*60;
% Number of simulation points
N_sim_pts=3001;
% Number of methylation sites
n_methyl=4;
% Cell volume [l]
CellVol=1.4e-15;

% Avogadro number
N_Av=6.02214179e23;

% Initial conditions definition
% Methylation and demethylation enzymes concentration [uM]
cheB_in=2000/CellVol/N_Av*1e6;
cheR_in=200/CellVol/N_Av*1e6;
% Ligand concentration [uM]
L_in=1;
% Unoccupied/occupied unmethylated/methylated 
% receptors complex concentrations [uM]
Etot=10000/CellVol/N_Av*1e6;

% Initial guess
Eu0_in=5000/CellVol/N_Av*1e6;
Eo0_in=5000/CellVol/N_Av*1e6;
Eu1_in=0/CellVol/N_Av*1e6;
Eo1_in=0/CellVol/N_Av*1e6;
Eu2_in=0/CellVol/N_Av*1e6;
Eo2_in=0/CellVol/N_Av*1e6;

% % Steady-state values for L_in=1e-5 M;
% Eu0_in=3.5096e-008;
% Eo0_in=3.5096e-007;
% Eu1_in=3.8571e-007;
% Eo1_in=3.9180e-006;
% Eu2_in=2.7523e-007;
% Eo2_in=2.9237e-006;

% Steady-state values for L_in=0 M;
% Eu0_in=1.9841e-006;
% Eo0_in=0;
% Eu1_in=1.9823e-006;
% Eo1_in=0;
% Eu2_in=3.9610e-007;
% Eo2_in=0;

x0=zeros(8+6*(n_methyl-1),1);
x0(1)=Eu0_in;
x0(2)=Eo0_in;
x0(5)=Eu1_in;
x0(6)=Eo1_in;
x0(11)=Eu2_in;
x0(12)=Eo2_in;

%
%load ss_L0 x_ss
%x0=x_ss;

% Simulation time
tv=linspace(0,Tf,N_sim_pts);
% Define input L(t)
%L=L_in*ones(N_sim_pts,1);
L=zeros(N_sim_pts,1);

%L(ceil(N_sim_pts/3):ceil(N_sim_pts/3*2))=4*L_in;

% Twenty minutes pulses of 1,3,5,7 uM ligand concentration every,
% interleaved by twenty minutes at zero concentration
L(ceil(N_sim_pts/9):ceil(N_sim_pts/9*2))=L_in;
L(ceil(N_sim_pts/9*3):ceil(N_sim_pts/9*4))=3*L_in;
L(ceil(N_sim_pts/9*5):ceil(N_sim_pts/9*6))=5*L_in;
L(ceil(N_sim_pts/9*7):ceil(N_sim_pts/9*8))=7*L_in;
load ss_L0 x_ss
x0=x_ss;

%[tout,xout]=ode23s(@ecoli_chemo,tv,x0,[],n_methyl,tv,L,cheB_in,cheR_in);
[tout,xout]=ode15s(@ecoli_chemo_v2,tv,x0(1:8+6*(n_methyl-1)-1),[],...
    Etot,n_methyl,tv,L,cheB_in,cheR_in);

% Activation probabilities
alph  = [0; 0.1; 0.5; 0.75; 1];
alpho = [0;   0; 0.1;  0.5; 1];

% Activity level of the system
Activ=0;
for i=1:n_methyl
    Activ=Activ+(alph(i+1)*xout(:,5+6*(i-1))...
        +alpho(i+1)*xout(:,6+6*(i-1)))*CellVol*N_Av/1e6;
end
% Normalize activity level (as percentage of steady-state level)
Activ=Activ/Activ(1)*100;

% Activ=(alph(2)*xout(:,5)+alpho(2)*xout(:,6)...
%     +alph(3)*xout(:,11)+alpho(3)*xout(:,12)...
%     +alph(4)*xout(:,17)+alpho(4)*xout(:,18)...
%     +alph(5)*xout(:,23)+alpho(5)*xout(:,24))*CellVol*N_Av/1e6;

% Change time scale to minutes
tout_min=tout/60;
figure
plot(tout_min,Activ);
ylabel('Activity [% of steady-state level]');
xlabel('Time [min]');

figure
i=1;
subplot(2,2,i);
plot(tout_min,xout(:,1)+xout(:,2));
ylabel('E_{unmeth} [uM]');
xlabel('Time [min]');
i=i+1;
subplot(2,2,i);
Em_tot=0;
for k=2:n_methyl
    Em_tot=Em_tot+xout(:,5+6*(k-1))+xout(:,6+6*(k-1));
end
plot(tout_min,Em_tot);
ylabel('E_{meth} [uM]');
xlabel('Time [min]');
i=i+1;
subplot(2,2,i);
Eu_tot=xout(:,1);
for k=1:n_methyl
    Eu_tot=Eu_tot+xout(:,5+6*(k-1));
end
plot(tout_min,Eu_tot);
ylabel('E^u [uM]');
xlabel('Time [min]');
i=i+1;
subplot(2,2,i);
Eo_tot=xout(:,2);
for k=1:n_methyl
    Eo_tot=Eo_tot+xout(:,6+6*(k-1));
end
plot(tout_min,Eo_tot);
ylabel('E^o [uM]');
xlabel('Time [min]');